==> smart-input-1.0.0.tgz: package/src/SmartInput.tsx <==
import * as React from 'react'
import { BaseEditor, createEditor, Node, Transforms } from 'slate'
import { withHistory } from 'slate-history'
import { DefaultElement, Editable, ReactEditor, Slate, withReact } from 'slate-react'

type CustomElement = {
  type: 'text' | 'completion' | undefined
  children: CustomText[]
}
type CustomText = { text: string; type: undefined }

type CustomDescendant = CustomElement | CustomText

type CEditor = Omit<BaseEditor, 'children'> & ReactEditor & { children: CustomDescendant[] }

const sessionSid = '' + Date.now()

declare module 'slate' {
  interface CustomTypes {
    Editor: CEditor
    Element: CustomElement
    Text: CustomText
  }
}

export interface SmartInputProps {
  modelApiKey: string
  value: string
  multiline?: boolean
  placeholder?: string
  onChange: (newValue: string) => void
  renderText: (props: any) => React.ReactElement
  renderCompletion: (props: any) => React.ReactElement
  onKeyDown?: React.KeyboardEventHandler<HTMLDivElement>
  onBlur?: React.FocusEventHandler<HTMLDivElement>
  onFocus?: React.FocusEventHandler<HTMLDivElement>
  className?: string
  style?: React.CSSProperties
  apiEndpoint?: string
}

export function SmartInput(props: SmartInputProps) {
  const editor = React.useMemo(() => withHistory(withReact(createEditor())), [])
  const [completion, setCompletion] = React.useState('')

  const initialValue: CustomDescendant[] = [
    {
      type: 'text',
      children: [{ text: props.value, type: undefined }],
    },
  ]

  const renderElement = React.useCallback(
    (p: any) => {
      switch (p.element.type) {
        case 'text':
          return props.renderText({
            ...p,
          })
        case 'completion':
          return props.renderCompletion({
            ...p,
            contentEditable: 'false',
            suppressContentEditableWarning: true,
          })
        default:
          return <DefaultElement {...p} />
      }
    },
    [props.renderText, props.renderCompletion]
  )

  const acceptSuggestion = () => {
    acceptCompletion(editor, completion)
    setCompletion('')
  }

  const addSuggestion = (completion: string) => {
    addCompletion(editor, completion)
    setCompletion(completion)
  }

  const clearSuggestion = () => {
    removeCompletion(editor)
    setCompletion('')
  }

  const sendEvents = React.useMemo(() => {
    return debounce((text: string, context: any): void => {
      fetch(props.apiEndpoint || 'https://api.lydian.company/app/events', {
        method: 'POST',
        mode: 'cors',
        headers: {
          'Content-Type': 'application/json',
          Authorization: `Bearer ${props.modelApiKey}`,
        },
        body: JSON.stringify({
          sessionSid,
          input: text,
          context,
        }),
      })
        .then(res => {
          if (res.status !== 200) {
            console.error("Couldn't fetch suggestions")
            return
          }
          return res.json()
        })
        .then(data => {
          const suggestion = data.suggestion
          if (suggestion) {
            addSuggestion(suggestion)
          }
        })
        .catch(e => console.error(e))
    }, 200)
  }, [props.modelApiKey])

  return (
    <Slate
      editor={editor}
      value={initialValue}
      onChange={value => {
        const text = serialize(value.filter(n => n.type !== 'completion'))
        sendEvents(text, {})
        props.onChange(text)
      }}
    >
      <Editable
        placeholder={props.placeholder}
        className={props.className}
        style={props.style}
        onPaste={(event: any) => {
          clearSuggestion()
          event.preventDefault()
          const value = event.clipboardData.getData('Text')
          if (!props.multiline) {
            editor.insertText(value.replaceAll('\n', ''))
          } else {
            editor.insertText(value)
          }
        }}
        renderElement={renderElement}
        onBlur={props.onBlur}
        onFocus={props.onFocus}
        onKeyDown={(event: any) => {
          const hasCompletion = completion.trim().length > 0

          if (event.key === 'Tab' && hasCompletion) {
            event.preventDefault()
            acceptSuggestion()
          } else if (event.key === 'ArrowRight' && hasCompletion) {
            event.preventDefault()
            acceptSuggestion()
          } else if (event.key === 'ArrowLeft' && hasCompletion) {
            clearSuggestion()
          } else if ((event.key === 'ArrowDown' || event.ke === 'ArrowUp') && hasCompletion) {
            clearSuggestion()
          } else if (event.key === 'Backspace') {
            clearSuggestion()
          } else if (event.key === 'Enter' && !event.metaKey) {
            event.preventDefault()
            if (props.multiline) {
              clearSuggestion()
              editor.insertText('\n')
            }
          } else {
            clearSuggestion() // TODO: this is a bit aggressive?
          }

          if (props.onKeyDown) {
            props.onKeyDown(event)
          }
        }}
      />
    </Slate>
  )
}

// Define helpers

function isCompletionActive(editor: CEditor) {
  const completion = editor.children.find(n => n.type === 'completion')
  return !!completion
}

function isCursorAtEndOfDocument(editor: CEditor) {
  const sel = editor.selection
  if (!sel) {
    return false
  }
  const [node] = sel.anchor.path
  const offset = sel.anchor.offset
  if (node !== editor.children.length - 1) {
    // not at the last node
    return false
  }
  return offset === Node.string(editor.children[node]).length
}

function acceptCompletion(editor: CEditor, completion: string) {
  const isActive = isCompletionActive(editor)
  if (isActive) {
    removeCompletion(editor)
    editor.insertText(completion)
  }
}

function addCompletion(editor: CEditor, completion: string) {
  const isActive = isCompletionActive(editor)
  const isCursorEnd = isCursorAtEndOfDocument(editor)
  if (!isActive && isCursorEnd) {
    Transforms.insertNodes(
      editor,
      {
        type: 'completion',
        children: [{ text: completion }],
      } as CustomDescendant,
      {
        at: [editor.children.length],
      }
    )
  }
}

function removeCompletion(editor: CEditor) {
  const isActive = isCompletionActive(editor)
  if (isActive) {
    Transforms.removeNodes(editor, {
      at: [editor.children.length - 1],
    })
  }
}

const serialize = (nodes: CustomDescendant[]) => {
  return nodes.map(n => Node.string(n)).join('\n')
}

const debounce = (callback: any, wait: number) => {
  let timeoutId: any = null
  return (...args: any[]) => {
    window.clearTimeout(timeoutId)
    timeoutId = window.setTimeout(() => {
      callback.apply(null, args)
    }, wait)
  }
}

==> smart-input-1.0.3.tgz: package/src/SmartInput.tsx <==
import * as React from 'react'
import { BaseEditor, createEditor, Node, Transforms } from 'slate'
import { withHistory } from 'slate-history'
import { DefaultElement, Editable, ReactEditor, Slate, withReact } from 'slate-react'

type CustomElement = {
  type: 'text' | 'completion' | undefined
  children: CustomText[]
}
type CustomText = { text: string; type: undefined }

type CustomDescendant = CustomElement | CustomText

type CEditor = Omit<BaseEditor, 'children'> & ReactEditor & { children: CustomDescendant[] }

const sessionSid = '' + Date.now()

declare module 'slate' {
  interface CustomTypes {
    Editor: CEditor
    Element: CustomElement
    Text: CustomText
  }
}

export interface SmartInputProps {
  modelId: string
  value: string
  onChange: (newValue: string) => void
  renderText: (props: any) => React.ReactElement
  renderSuggestion: (props: any) => React.ReactElement
  className?: string
  multiline?: boolean
  onBlur?: React.FocusEventHandler<HTMLDivElement>
  onFocus?: React.FocusEventHandler<HTMLDivElement>
  onKeyDown?: React.KeyboardEventHandler<HTMLDivElement>
  placeholder?: string
  style?: React.CSSProperties
  apiEndpoint?: string
}

export function SmartInput(props: SmartInputProps) {
  const editor = React.useMemo(() => withHistory(withReact(createEditor())), [])
  const [completion, setCompletion] = React.useState('')

  const initialValue: CustomDescendant[] = [
    {
      type: 'text',
      children: [{ text: props.value, type: undefined }],
    },
  ]

  const renderElement = React.useCallback(
    (p: any) => {
      switch (p.element.type) {
        case 'text':
          return props.renderText({
            ...p,
          })
        case 'completion':
          return props.renderSuggestion({
            ...p,
            contentEditable: 'false',
            suppressContentEditableWarning: true,
          })
        default:
          return <DefaultElement {...p} />
      }
    },
    [props.renderText, props.renderSuggestion]
  )

  const acceptSuggestion = () => {
    acceptCompletion(editor, completion)
    setCompletion('')
  }

  const addSuggestion = (completion: string) => {
    addCompletion(editor, completion)
    setCompletion(completion)
  }

  const clearSuggestion = () => {
    removeCompletion(editor)
    setCompletion('')
  }

  const sendEvents = React.useMemo(() => {
    return debounce((text: string, context: any): void => {
      fetch(props.apiEndpoint || 'https://api.lydian.company/app/events', {
        method: 'POST',
        mode: 'cors',
        headers: {
          'Content-Type': 'application/json',
          Authorization: `Bearer ${props.modelId}`,
        },
        body: JSON.stringify({
          model: props.modelId,
          session: sessionSid,
          input: text,
          context,
        }),
      })
        .then(res => {
          if (res.status !== 200) {
            console.error("Couldn't fetch suggestions")
            return
          }
          return res.json()
        })
        .then(data => {
          const suggestion = data.suggestion
          if (suggestion) {
            addSuggestion(suggestion)
          }
        })
        .catch(e => console.error(e))
    }, 200)
  }, [props.modelId])

  return (
    <Slate
      editor={editor}
      value={initialValue}
      onChange={value => {
        const text = serialize(value.filter(n => n.type !== 'completion'))
        sendEvents(text, {})
        props.onChange(text)
      }}
    >
      <Editable
        placeholder={props.placeholder}
        className={props.className}
        style={props.style}
        onPaste={(event: any) => {
          clearSuggestion()
          event.preventDefault()
          const value = event.clipboardData.getData('Text')
          if (!props.multiline) {
            editor.insertText(value.replaceAll('\n', ''))
          } else {
            editor.insertText(value)
          }
        }}
        renderElement={renderElement}
        onBlur={props.onBlur}
        onFocus={props.onFocus}
        onKeyDown={(event: any) => {
          const hasCompletion = completion.trim().length > 0
          if (event.key === 'Tab' && hasCompletion) {
            event.preventDefault()
            acceptSuggestion()
          } else if (event.key === 'ArrowRight' && hasCompletion) {
            event.preventDefault()
            acceptSuggestion()
          } else if (event.key === 'ArrowLeft' && hasCompletion) {
            clearSuggestion()
          } else if ((event.key === 'ArrowDown' || event.ke === 'ArrowUp') && hasCompletion) {
            clearSuggestion()
          } else if (event.key === 'Backspace') {
            clearSuggestion()
          } else if (event.key === 'Enter' && !event.metaKey) {
            event.preventDefault()
            if (props.multiline) {
              clearSuggestion()
              editor.insertText('\n')
            }
          } else {
            clearSuggestion() // TODO: this is a bit aggressive?
          }

          if (props.onKeyDown) {
            props.onKeyDown(event)
          }
        }}
      />
    </Slate>
  )
}

// Define helpers

function isCompletionActive(editor: CEditor) {
  const completion = editor.children.find(n => n.type === 'completion')
  return !!completion
}

function isCursorAtEndOfDocument(editor: CEditor) {
  const sel = editor.selection
  if (!sel) {
    return false
  }
  const [node] = sel.anchor.path
  const offset = sel.anchor.offset
  if (node !== editor.children.length - 1) {
    // not at the last node
    return false
  }
  return offset === Node.string(editor.children[node]).length
}

function acceptCompletion(editor: CEditor, completion: string) {
  const isActive = isCompletionActive(editor)
  if (isActive) {
    removeCompletion(editor)
    editor.insertText(completion)
  }
}

function addCompletion(editor: CEditor, completion: string) {
  const isActive = isCompletionActive(editor)
  const isCursorEnd = isCursorAtEndOfDocument(editor)
  if (!isActive && isCursorEnd) {
    Transforms.insertNodes(
      editor,
      {
        type: 'completion',
        children: [{ text: completion }],
      } as CustomDescendant,
      {
        at: [editor.children.length],
      }
    )
  }
}

function removeCompletion(editor: CEditor) {
  const isActive = isCompletionActive(editor)
  if (isActive) {
    Transforms.removeNodes(editor, {
      at: [editor.children.length - 1],
    })
  }
}

const serialize = (nodes: CustomDescendant[]) => {
  return nodes.map(n => Node.string(n)).join('\n')
}

const debounce = (callback: any, wait: number) => {
  let timeoutId: any = null
  return (...args: any[]) => {
    window.clearTimeout(timeoutId)
    timeoutId = window.setTimeout(() => {
      callback.apply(null, args)
    }, wait)
  }
}
